package CUED::EDDA::Person;
use base CUED::EDDA::DBIC;
use CUED::EDDA::College;

use warnings;
use strict;

=head1 NAME

CUED::EDDA::Person - The great new CUED::EDDA::Person!

=head1 VERSION

Version 0.01

=cut

our $VERSION = '0.01';

=head1 SYNOPSIS

Retrieves info about people from EDDA.

    use CUED::EDDA::Person;

    my $p = EDDA::Person->by_pk('mjg17');     # find by personnel_key
    my $p = EDDA::Person->by_crsid('mjg17');  # find by CRSid
    my $p = EDDA::Person->by_engid('pnt');    # find by engineering id

    print $p->surname();
    print $p->class();

    # can provide full class or college names

    print $p->longclass();
    print $p->longcoll();

    # Various methods return either undef or the record from the
    # associated view

    if ($p->staff()) {
      my $staff = $p->staff[0];	# may be multiple staff records
      print $staff->job_title();
    }

    # Ugrad just says that person is some kind of ugrad - simple wrapper
    # on the results of $p->student().

    if ($p->ugrad()) {
      print $p->student()->usn();
      if ($p->ugrad1()) {
        print $p->ugrad1()->labgroup1();
      }
    }

    # Also

    $p->collaborator()
    $p->student()
    $p->pgrad()
    $p->visitor()
    $p->other()
    $p->retired()
    $p->teaching_collaborator()  # just returns true if
                                 # class is tcollaborator


=head1 CONSTRUCTORS

=head2 by_pk

Finds person record by personnel key

    my $p = EDDA::Person->by_pk('mjg17');

=head2 by_crsid

Finds person record by CRSID

    my $p = EDDA::Person->by_crsid('mjg17');

=head2 by_engid

Finds person record by ENGID

    my $p = EDDA::Person->by_engid('mjg17');

=cut

sub resultset {
    my $self = shift;
    return $self->schema()->resultset('Person');
}

# by_pk is inherited from DBIC.pm

sub by_crsid {
    my ($proto,$crsid) = @_;
    my $class = ref($proto) || $proto;    
    my @p = $class->resultset()->search({crsid => $crsid});
    return $p[0];
}

sub by_engid {
    my ($proto,$engid) = @_;
    my $class = ref($proto) || $proto;
    my @p = $class->resultset()->search({engid => $engid});
    return $p[0];
}

=head1 METHODS

=head2 by_surname

Fetches all person records matching the given surname
    my @p = CUED::EDDA::Person->by_surname("Jones");
    foreach my $p (@p) {
        print $p->personnel_key;
        print $p->surname;
    }

=head2 collaborator

Returns collaborator record if it exists, or undef otherwise

    if ($collab = $p->collaborator) {
        print $collab->sponsoring_staff_personnel_key;
    }

=head2 other

Returns other record if it exists, or undef otherwise

    if ($other = $p->other) {
        print $other->organisation;
    }


=head2 retired, pgrad, visitor, ugrad1, ugrad2, ugrad3, ugrad4

Returns relevant record if it exist, or undef otherwise, as collaborator or other

    if ($retired = $p->retired) {
        print $retired->date_retired;
    }

=head2 staff

Warning, as there may be more than one staff record per person this returns a list of staff records, rather than one record

    @staff = $p->staff;
    foreach $staff (@staff) {
        print $staff->job_title;
    }
    

=head2 ugrad

Returns 1 if ugrad and 0 otherwise

    if ($p->ugrad()) {
      print $p->student()->usn();
      if ($p->ugrad1()) {
        print $p->ugrad1()->labgroup1();
      }
    }

=head2 tcollaborator

Returns 1 if tcollaborator and 0 otherwise

    if ($person->tcollaborator) {
       print "$person->personnel_key is a Teaching Collaborator\n";
   }

=head2 fullname

Works out and returns the full preferred name, from available data

    print $p->fullname;

=head2 longclass

Turns the class of the person into the long version for printing
    
    print $p->longclass;

=head2 longcoll

Turns the college (if available) into full name

    if ($p->college) {
       print $p->longcoll;
    }

=cut

sub by_surname() {
    my ($proto,$surname) = @_;
    my $class = ref($proto) || $proto;
    my @p = $class->resultset()->search({surname => $surname});
    return @p;
}
    
=head1 AUTHOR

Michael Gray, C<< <mjg17 at eng.cam.ac.uk> >>

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc CUED::EDDA::Person

=head1 ACKNOWLEDGEMENTS

=head1 COPYRIGHT & LICENSE

Copyright 2006 Michael Gray, all rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

1; # End of CUED::EDDA::Person
